-- User implementation of the associatedcheckstates function
-- This file will never be overwritten once edited and modified
-- Only the interface of functions is regenerated (in the .ads file)

pragma style_checks (off);
pragma warnings (off);
with adaasn1rtl;
use adaasn1rtl;

with onboard_parameters;
use onboard_parameters;

with onboard_monitoring_dataview;
use onboard_monitoring_dataview;

with Standard_Datatypes;
use Standard_Datatypes;

with onboard_monitoring;
use onboard_monitoring;

package body associatedcheckstates is


   -- Arbitrary Parameter Value
   DefaultParameterValue : constant asn1SccParameter_Value := (kind     => type_1_0_PRESENT,
                                                               type_1_0 => asn1SccUnsigned_Integer'first);

   -- Initial check state associated until first actual check takes place
   InitialCheckState : constant asn1sccCheck_State := (
     id                       => asn1SccParameter_ID'first,
     value                    => DefaultParameterValue,
     limit_crossed            => DefaultParameterValue,
     previous_checking_status => asn1Sccunchecked,
     current_checking_status  => asn1Sccunchecked,
     transition_time          => asn1SccOn_Board_Time'(p_field => (0, 0),
                                                       coarse_time => (data => (0,0,0,0)),
                                                       fine_time => (data => (0,0,0))
      )
    );

   -- Simple storage of check states based on linear search in array
   type CheckStateMap is array (asn1SccParameter_ID) of asn1sccCheck_State;
   theAssociatedStates : CheckStateMap := (others => InitialCheckState);

   ---------------------------------------------------------
   -- Provided interface "UpdateCheckState"
   ---------------------------------------------------------
   procedure UpdateCheckState (id : access asn1sccParameter_ID; state : access asn1sccCheck_State) is
   begin

      theAssociatedStates (id.all) := state.all;

   end UpdateCheckState;

   ---------------------------------------------------------
   -- Provided interface "GetCheckState"
   ---------------------------------------------------------
   procedure GetCheckState (id : access asn1sccParameter_ID; state : access asn1sccCheck_State) is
   begin

      state.all := theAssociatedStates (id.all);

   end GetCheckState;


end associatedcheckstates;
