#include <stdio.h>

#include "PrintTypesAsASN1.h"

#ifdef __linux__
#include <pthread.h>

static pthread_mutex_t g_printing_mutex = PTHREAD_MUTEX_INITIALIZER;

#endif

void PrintASN1Event_Action_Service_Report(const char *paramName, const asn1SccEvent_Action_Service_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Event-Action-Service-Report ::= ", paramName);
    printf("%s ", paramName);
    if ((*pData).kind == foo_PRESENT) {
        printf("foo:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.foo);
        #else
        printf("%d", (*pData).u.foo);
        #endif
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_11(const char *paramName, const asn1SccFixed_Octet_String_11 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-11 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<11; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Standard_Boolean(const char *paramName, const asn1SccStandard_Boolean *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Standard-Boolean ::= ", paramName);
    printf("%s ", paramName);
    printf("%s", (int)(*pData)?"TRUE":"FALSE");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_15(const char *paramName, const asn1SccFixed_Character_String_15 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-15 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<15; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_14(const char *paramName, const asn1SccFixed_Character_String_14 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-14 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<14; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_10(const char *paramName, const asn1SccFixed_Character_String_10 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-10 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<10; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Signed_Integer(const char *paramName, const asn1SccSigned_Integer *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Signed-Integer ::= ", paramName);
    printf("%s ", paramName);
    #if WORD_SIZE==8
    printf("%lld", (*pData));
    #else
    printf("%d", (*pData));
    #endif
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Low_Severity_Error_Anomaly_Report(const char *paramName, const asn1SccLow_Severity_Error_Anomaly_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Low-Severity-Error-Anomaly-Report ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("rid ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).rid);
    #else
    printf("%d", (*pData).rid);
    #endif
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Telecommand_Verification_Service_Report(const char *paramName, const asn1SccTelecommand_Verification_Service_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Telecommand-Verification-Service-Report ::= ", paramName);
    printf("%s ", paramName);
    if ((*pData).kind == success_PRESENT) {
        printf("success:");
        printf("{");
        printf("packet-id ");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.success.packet_id);
        #else
        printf("%d", (*pData).u.success.packet_id);
        #endif
        printf("}");
    }
    else if ((*pData).kind == failure_PRESENT) {
        printf("failure:");
        printf("{");
        printf("packet-id ");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.failure.packet_id);
        #else
        printf("%d", (*pData).u.failure.packet_id);
        #endif
        printf(", ");
        printf("code ");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.failure.code);
        #else
        printf("%d", (*pData).u.failure.code);
        #endif
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_16(const char *paramName, const asn1SccFixed_Character_String_16 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-16 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<16; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Report_ID(const char *paramName, const asn1SccReport_ID *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Report-ID ::= ", paramName);
    printf("%s ", paramName);
    #if WORD_SIZE==8
    printf("%lld", (*pData));
    #else
    printf("%d", (*pData));
    #endif
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1On_Board_Time(const char *paramName, const asn1SccOn_Board_Time *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s On-Board-Time ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("p-field ");
    printf("{");
    printf("coarse-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).p_field.coarse_time_rep);
    #else
    printf("%d", (*pData).p_field.coarse_time_rep);
    #endif
    printf(", ");
    printf("fine-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).p_field.fine_time_rep);
    #else
    printf("%d", (*pData).p_field.fine_time_rep);
    #endif
    printf("}");
    printf(", ");
    printf("coarse-time ");
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).coarse_time.arr[i]);
        printf("'H");
    }

    printf(", ");
    printf("fine-time ");
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).fine_time.arr[i]);
        printf("'H");
    }

    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_2(const char *paramName, const asn1SccFixed_Bit_String_2 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-2 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i1;
        printf("{");
        for(i1=0; i1<2; i1++) {
            if (i1) 
                printf(",");
            printf("%s", (int)(*pData).arr[i1]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Event_Reporting_Service_Report(const char *paramName, const asn1SccEvent_Reporting_Service_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Event-Reporting-Service-Report ::= ", paramName);
    printf("%s ", paramName);
    if ((*pData).kind == normal_PRESENT) {
        printf("normal:");
        printf("{");
        printf("rid ");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.normal.rid);
        #else
        printf("%d", (*pData).u.normal.rid);
        #endif
        printf("}");
    }
    else if ((*pData).kind == anomaly_PRESENT) {
        printf("anomaly:");
        if ((*pData).u.anomaly.kind == low_PRESENT) {
            printf("low:");
            printf("{");
            printf("rid ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.anomaly.u.low.rid);
            #else
            printf("%d", (*pData).u.anomaly.u.low.rid);
            #endif
            printf("}");
        }
        else if ((*pData).u.anomaly.kind == medium_PRESENT) {
            printf("medium:");
            printf("{");
            printf("rid ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.anomaly.u.medium.rid);
            #else
            printf("%d", (*pData).u.anomaly.u.medium.rid);
            #endif
            printf("}");
        }
        else if ((*pData).u.anomaly.kind == high_PRESENT) {
            printf("high:");
            printf("{");
            printf("rid ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.anomaly.u.high.rid);
            #else
            printf("%d", (*pData).u.anomaly.u.high.rid);
            #endif
            printf("}");
        }
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_10(const char *paramName, const asn1SccFixed_Octet_String_10 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-10 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<10; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_13(const char *paramName, const asn1SccFixed_Octet_String_13 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-13 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<13; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_12(const char *paramName, const asn1SccFixed_Octet_String_12 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-12 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<12; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_15(const char *paramName, const asn1SccFixed_Octet_String_15 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-15 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<15; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_14(const char *paramName, const asn1SccFixed_Octet_String_14 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-14 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<14; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Telecommand_Packet_ID(const char *paramName, const asn1SccTelecommand_Packet_ID *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Telecommand-Packet-ID ::= ", paramName);
    printf("%s ", paramName);
    #if WORD_SIZE==8
    printf("%lld", (*pData));
    #else
    printf("%d", (*pData));
    #endif
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_11(const char *paramName, const asn1SccFixed_Character_String_11 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-11 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<11; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_9(const char *paramName, const asn1SccFixed_Octet_String_9 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-9 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<9; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_8(const char *paramName, const asn1SccFixed_Octet_String_8 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-8 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<8; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Parameter_Value(const char *paramName, const asn1SccParameter_Value *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Parameter-Value ::= ", paramName);
    printf("%s ", paramName);
    if ((*pData).kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.type_1_0);
        #else
        printf("%d", (*pData).u.type_1_0);
        #endif
    }
    else if ((*pData).kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.type_1_1);
        #else
        printf("%d", (*pData).u.type_1_1);
        #endif
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_4(const char *paramName, const asn1SccFixed_Character_String_4 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-4 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_1(const char *paramName, const asn1SccFixed_Octet_String_1 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-1 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<1; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_3(const char *paramName, const asn1SccFixed_Octet_String_3 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-3 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Check_State(const char *paramName, const asn1SccCheck_State *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Check-State ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("id ");
    switch((*pData).id) {
    case 0:
        printf("voltage-1");
        break;
    case 1:
        printf("voltage-2");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).id)");
    }
    printf(", ");
    printf("value ");
    if ((*pData).value.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).value.u.type_1_0);
        #else
        printf("%d", (*pData).value.u.type_1_0);
        #endif
    }
    else if ((*pData).value.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).value.u.type_1_1);
        #else
        printf("%d", (*pData).value.u.type_1_1);
        #endif
    }
    printf(", ");
    printf("limit-crossed ");
    if ((*pData).limit_crossed.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).limit_crossed.u.type_1_0);
        #else
        printf("%d", (*pData).limit_crossed.u.type_1_0);
        #endif
    }
    else if ((*pData).limit_crossed.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).limit_crossed.u.type_1_1);
        #else
        printf("%d", (*pData).limit_crossed.u.type_1_1);
        #endif
    }
    printf(", ");
    printf("previous-checking-status ");
    switch((*pData).previous_checking_status) {
    case 0:
        printf("witin-limits");
        break;
    case 1:
        printf("unchecked");
        break;
    case 4:
        printf("below-low-limit");
        break;
    case 5:
        printf("above-high-limit");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).previous_checking_status)");
    }
    printf(", ");
    printf("current-checking-status ");
    switch((*pData).current_checking_status) {
    case 0:
        printf("witin-limits");
        break;
    case 1:
        printf("unchecked");
        break;
    case 4:
        printf("below-low-limit");
        break;
    case 5:
        printf("above-high-limit");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).current_checking_status)");
    }
    printf(", ");
    printf("transition-time ");
    printf("{");
    printf("p-field ");
    printf("{");
    printf("coarse-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).transition_time.p_field.coarse_time_rep);
    #else
    printf("%d", (*pData).transition_time.p_field.coarse_time_rep);
    #endif
    printf(", ");
    printf("fine-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).transition_time.p_field.fine_time_rep);
    #else
    printf("%d", (*pData).transition_time.p_field.fine_time_rep);
    #endif
    printf("}");
    printf(", ");
    printf("coarse-time ");
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).transition_time.coarse_time.arr[i]);
        printf("'H");
    }

    printf(", ");
    printf("fine-time ");
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).transition_time.fine_time.arr[i]);
        printf("'H");
    }

    printf("}");
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_5(const char *paramName, const asn1SccFixed_Octet_String_5 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-5 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<5; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_4(const char *paramName, const asn1SccFixed_Octet_String_4 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-4 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_7(const char *paramName, const asn1SccFixed_Octet_String_7 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-7 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<7; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_6(const char *paramName, const asn1SccFixed_Octet_String_6 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-6 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<6; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_6(const char *paramName, const asn1SccFixed_Character_String_6 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-6 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<6; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_9(const char *paramName, const asn1SccFixed_Bit_String_9 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-9 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i2;
        printf("{");
        for(i2=0; i2<9; i2++) {
            if (i2) 
                printf(",");
            printf("%s", (int)(*pData).arr[i2]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_8(const char *paramName, const asn1SccFixed_Bit_String_8 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-8 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i3;
        printf("{");
        for(i3=0; i3<8; i3++) {
            if (i3) 
                printf(",");
            printf("%s", (int)(*pData).arr[i3]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_7(const char *paramName, const asn1SccFixed_Bit_String_7 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-7 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i4;
        printf("{");
        for(i4=0; i4<7; i4++) {
            if (i4) 
                printf(",");
            printf("%s", (int)(*pData).arr[i4]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_6(const char *paramName, const asn1SccFixed_Bit_String_6 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-6 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i5;
        printf("{");
        for(i5=0; i5<6; i5++) {
            if (i5) 
                printf(",");
            printf("%s", (int)(*pData).arr[i5]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_5(const char *paramName, const asn1SccFixed_Bit_String_5 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-5 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i6;
        printf("{");
        for(i6=0; i6<5; i6++) {
            if (i6) 
                printf(",");
            printf("%s", (int)(*pData).arr[i6]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_4(const char *paramName, const asn1SccFixed_Bit_String_4 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-4 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i7;
        printf("{");
        for(i7=0; i7<4; i7++) {
            if (i7) 
                printf(",");
            printf("%s", (int)(*pData).arr[i7]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_3(const char *paramName, const asn1SccFixed_Bit_String_3 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-3 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i8;
        printf("{");
        for(i8=0; i8<3; i8++) {
            if (i8) 
                printf(",");
            printf("%s", (int)(*pData).arr[i8]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Medium_Severity_Error_Anomaly_Report(const char *paramName, const asn1SccMedium_Severity_Error_Anomaly_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Medium-Severity-Error-Anomaly-Report ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("rid ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).rid);
    #else
    printf("%d", (*pData).rid);
    #endif
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_13(const char *paramName, const asn1SccFixed_Character_String_13 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-13 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<13; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_12(const char *paramName, const asn1SccFixed_Character_String_12 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-12 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<12; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_5(const char *paramName, const asn1SccFixed_Character_String_5 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-5 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<5; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Transition_Report(const char *paramName, const asn1SccTransition_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Transition-Report ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("id ");
    switch((*pData).id) {
    case 0:
        printf("voltage-1");
        break;
    case 1:
        printf("voltage-2");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).id)");
    }
    printf(", ");
    printf("value ");
    if ((*pData).value.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).value.u.type_1_0);
        #else
        printf("%d", (*pData).value.u.type_1_0);
        #endif
    }
    else if ((*pData).value.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).value.u.type_1_1);
        #else
        printf("%d", (*pData).value.u.type_1_1);
        #endif
    }
    printf(", ");
    printf("limit-crossed ");
    if ((*pData).limit_crossed.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).limit_crossed.u.type_1_0);
        #else
        printf("%d", (*pData).limit_crossed.u.type_1_0);
        #endif
    }
    else if ((*pData).limit_crossed.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).limit_crossed.u.type_1_1);
        #else
        printf("%d", (*pData).limit_crossed.u.type_1_1);
        #endif
    }
    printf(", ");
    printf("previous-checking-status ");
    switch((*pData).previous_checking_status) {
    case 0:
        printf("witin-limits");
        break;
    case 1:
        printf("unchecked");
        break;
    case 4:
        printf("below-low-limit");
        break;
    case 5:
        printf("above-high-limit");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).previous_checking_status)");
    }
    printf(", ");
    printf("current-checking-status ");
    switch((*pData).current_checking_status) {
    case 0:
        printf("witin-limits");
        break;
    case 1:
        printf("unchecked");
        break;
    case 4:
        printf("below-low-limit");
        break;
    case 5:
        printf("above-high-limit");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).current_checking_status)");
    }
    printf(", ");
    printf("transition-time ");
    printf("{");
    printf("p-field ");
    printf("{");
    printf("coarse-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).transition_time.p_field.coarse_time_rep);
    #else
    printf("%d", (*pData).transition_time.p_field.coarse_time_rep);
    #endif
    printf(", ");
    printf("fine-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).transition_time.p_field.fine_time_rep);
    #else
    printf("%d", (*pData).transition_time.p_field.fine_time_rep);
    #endif
    printf("}");
    printf(", ");
    printf("coarse-time ");
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).transition_time.coarse_time.arr[i]);
        printf("'H");
    }

    printf(", ");
    printf("fine-time ");
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).transition_time.fine_time.arr[i]);
        printf("'H");
    }

    printf("}");
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_7(const char *paramName, const asn1SccFixed_Character_String_7 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-7 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<7; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Var_Bit_String(const char *paramName, const asn1SccVar_Bit_String *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Var-Bit-String ::= ", paramName);
    printf("%s ", paramName);
    {
        int i9;
        printf("{");
        for(i9=0; i9<(*pData).nCount; i9++) {
            if (i9) 
                printf(",");
            printf("%s", (int)(*pData).arr[i9]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_1(const char *paramName, const asn1SccFixed_Character_String_1 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-1 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<1; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1High_Severity_Error_Anomaly_Report(const char *paramName, const asn1SccHigh_Severity_Error_Anomaly_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s High-Severity-Error-Anomaly-Report ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("rid ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).rid);
    #else
    printf("%d", (*pData).rid);
    #endif
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Check_Definition(const char *paramName, const asn1SccCheck_Definition *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Check-Definition ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("lower-bound ");
    if ((*pData).lower_bound.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).lower_bound.u.type_1_0);
        #else
        printf("%d", (*pData).lower_bound.u.type_1_0);
        #endif
    }
    else if ((*pData).lower_bound.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).lower_bound.u.type_1_1);
        #else
        printf("%d", (*pData).lower_bound.u.type_1_1);
        #endif
    }
    printf(", ");
    printf("upper-bound ");
    if ((*pData).upper_bound.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).upper_bound.u.type_1_0);
        #else
        printf("%d", (*pData).upper_bound.u.type_1_0);
        #endif
    }
    else if ((*pData).upper_bound.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).upper_bound.u.type_1_1);
        #else
        printf("%d", (*pData).upper_bound.u.type_1_1);
        #endif
    }
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_2(const char *paramName, const asn1SccFixed_Character_String_2 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-2 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<2; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_9(const char *paramName, const asn1SccFixed_Character_String_9 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-9 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<9; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_8(const char *paramName, const asn1SccFixed_Character_String_8 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-8 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<8; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Onboard_Monitoring_Service_Report(const char *paramName, const asn1SccOnboard_Monitoring_Service_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Onboard-Monitoring-Service-Report ::= ", paramName);
    printf("%s ", paramName);
    if ((*pData).kind == transition_PRESENT) {
        printf("transition:");
        {
            int i10;
            printf("{");
            for(i10=0; i10<(*pData).u.transition.nCount; i10++) {
                if (i10) 
                    printf(",");
                printf("{");
                printf("id ");
                switch((*pData).u.transition.arr[i10].id) {
                case 0:
                    printf("voltage-1");
                    break;
                case 1:
                    printf("voltage-2");
                    break;
                default:
                    printf("Invalid value in ENUMERATED ((*pData).u.transition.arr[i10].id)");
                }
                printf(", ");
                printf("value ");
                if ((*pData).u.transition.arr[i10].value.kind == type_1_0_PRESENT) {
                    printf("type-1-0:");
                    #if WORD_SIZE==8
                    printf("%lld", (*pData).u.transition.arr[i10].value.u.type_1_0);
                    #else
                    printf("%d", (*pData).u.transition.arr[i10].value.u.type_1_0);
                    #endif
                }
                else if ((*pData).u.transition.arr[i10].value.kind == type_1_1_PRESENT) {
                    printf("type-1-1:");
                    #if WORD_SIZE==8
                    printf("%lld", (*pData).u.transition.arr[i10].value.u.type_1_1);
                    #else
                    printf("%d", (*pData).u.transition.arr[i10].value.u.type_1_1);
                    #endif
                }
                printf(", ");
                printf("limit-crossed ");
                if ((*pData).u.transition.arr[i10].limit_crossed.kind == type_1_0_PRESENT) {
                    printf("type-1-0:");
                    #if WORD_SIZE==8
                    printf("%lld", (*pData).u.transition.arr[i10].limit_crossed.u.type_1_0);
                    #else
                    printf("%d", (*pData).u.transition.arr[i10].limit_crossed.u.type_1_0);
                    #endif
                }
                else if ((*pData).u.transition.arr[i10].limit_crossed.kind == type_1_1_PRESENT) {
                    printf("type-1-1:");
                    #if WORD_SIZE==8
                    printf("%lld", (*pData).u.transition.arr[i10].limit_crossed.u.type_1_1);
                    #else
                    printf("%d", (*pData).u.transition.arr[i10].limit_crossed.u.type_1_1);
                    #endif
                }
                printf(", ");
                printf("previous-checking-status ");
                switch((*pData).u.transition.arr[i10].previous_checking_status) {
                case 0:
                    printf("witin-limits");
                    break;
                case 1:
                    printf("unchecked");
                    break;
                case 4:
                    printf("below-low-limit");
                    break;
                case 5:
                    printf("above-high-limit");
                    break;
                default:
                    printf("Invalid value in ENUMERATED ((*pData).u.transition.arr[i10].previous_checking_status)");
                }
                printf(", ");
                printf("current-checking-status ");
                switch((*pData).u.transition.arr[i10].current_checking_status) {
                case 0:
                    printf("witin-limits");
                    break;
                case 1:
                    printf("unchecked");
                    break;
                case 4:
                    printf("below-low-limit");
                    break;
                case 5:
                    printf("above-high-limit");
                    break;
                default:
                    printf("Invalid value in ENUMERATED ((*pData).u.transition.arr[i10].current_checking_status)");
                }
                printf(", ");
                printf("transition-time ");
                printf("{");
                printf("p-field ");
                printf("{");
                printf("coarse-time-rep ");
                #if WORD_SIZE==8
                printf("%lld", (*pData).u.transition.arr[i10].transition_time.p_field.coarse_time_rep);
                #else
                printf("%d", (*pData).u.transition.arr[i10].transition_time.p_field.coarse_time_rep);
                #endif
                printf(", ");
                printf("fine-time-rep ");
                #if WORD_SIZE==8
                printf("%lld", (*pData).u.transition.arr[i10].transition_time.p_field.fine_time_rep);
                #else
                printf("%d", (*pData).u.transition.arr[i10].transition_time.p_field.fine_time_rep);
                #endif
                printf("}");
                printf(", ");
                printf("coarse-time ");
                {
                    int i;
                    printf("'");
                    for(i=0; i<4; i++)
                        printf("%02x", (*pData).u.transition.arr[i10].transition_time.coarse_time.arr[i]);
                    printf("'H");
                }

                printf(", ");
                printf("fine-time ");
                {
                    int i;
                    printf("'");
                    for(i=0; i<3; i++)
                        printf("%02x", (*pData).u.transition.arr[i10].transition_time.fine_time.arr[i]);
                    printf("'H");
                }

                printf("}");
                printf("}");
            }
            printf("}");
        }
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_2(const char *paramName, const asn1SccFixed_Octet_String_2 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-2 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<2; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Pus_Relative_Time(const char *paramName, const asn1SccPus_Relative_Time *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Pus-Relative-Time ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("p-field ");
    printf("{");
    printf("coarse-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).p_field.coarse_time_rep);
    #else
    printf("%d", (*pData).p_field.coarse_time_rep);
    #endif
    printf(", ");
    printf("fine-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).p_field.fine_time_rep);
    #else
    printf("%d", (*pData).p_field.fine_time_rep);
    #endif
    printf("}");
    printf(", ");
    printf("coarse-time ");
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).coarse_time.arr[i]);
        printf("'H");
    }

    printf(", ");
    printf("fine-time ");
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).fine_time.arr[i]);
        printf("'H");
    }

    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Application_Process_ID(const char *paramName, const asn1SccApplication_Process_ID *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Application-Process-ID ::= ", paramName);
    printf("%s ", paramName);
    switch((*pData)) {
    case 0:
        printf("time-packet");
        break;
    case 1:
        printf("ground-if");
        break;
    case 4:
        printf("unit-manager");
        break;
    case 5:
        printf("packet-router-cdmu");
        break;
    case 2047:
        printf("idle");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData))");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Pus_Time(const char *paramName, const asn1SccPus_Time *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Pus-Time ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("p-field ");
    printf("{");
    printf("coarse-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).p_field.coarse_time_rep);
    #else
    printf("%d", (*pData).p_field.coarse_time_rep);
    #endif
    printf(", ");
    printf("fine-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).p_field.fine_time_rep);
    #else
    printf("%d", (*pData).p_field.fine_time_rep);
    #endif
    printf("}");
    printf(", ");
    printf("coarse-time ");
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).coarse_time.arr[i]);
        printf("'H");
    }

    printf(", ");
    printf("fine-time ");
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).fine_time.arr[i]);
        printf("'H");
    }

    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Unsigned_Integer(const char *paramName, const asn1SccUnsigned_Integer *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Unsigned-Integer ::= ", paramName);
    printf("%s ", paramName);
    #if WORD_SIZE==8
    printf("%lld", (*pData));
    #else
    printf("%d", (*pData));
    #endif
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Check_Transition_Report(const char *paramName, const asn1SccCheck_Transition_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Check-Transition-Report ::= ", paramName);
    printf("%s ", paramName);
    {
        int i11;
        printf("{");
        for(i11=0; i11<(*pData).nCount; i11++) {
            if (i11) 
                printf(",");
            printf("{");
            printf("id ");
            switch((*pData).arr[i11].id) {
            case 0:
                printf("voltage-1");
                break;
            case 1:
                printf("voltage-2");
                break;
            default:
                printf("Invalid value in ENUMERATED ((*pData).arr[i11].id)");
            }
            printf(", ");
            printf("value ");
            if ((*pData).arr[i11].value.kind == type_1_0_PRESENT) {
                printf("type-1-0:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i11].value.u.type_1_0);
                #else
                printf("%d", (*pData).arr[i11].value.u.type_1_0);
                #endif
            }
            else if ((*pData).arr[i11].value.kind == type_1_1_PRESENT) {
                printf("type-1-1:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i11].value.u.type_1_1);
                #else
                printf("%d", (*pData).arr[i11].value.u.type_1_1);
                #endif
            }
            printf(", ");
            printf("limit-crossed ");
            if ((*pData).arr[i11].limit_crossed.kind == type_1_0_PRESENT) {
                printf("type-1-0:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i11].limit_crossed.u.type_1_0);
                #else
                printf("%d", (*pData).arr[i11].limit_crossed.u.type_1_0);
                #endif
            }
            else if ((*pData).arr[i11].limit_crossed.kind == type_1_1_PRESENT) {
                printf("type-1-1:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i11].limit_crossed.u.type_1_1);
                #else
                printf("%d", (*pData).arr[i11].limit_crossed.u.type_1_1);
                #endif
            }
            printf(", ");
            printf("previous-checking-status ");
            switch((*pData).arr[i11].previous_checking_status) {
            case 0:
                printf("witin-limits");
                break;
            case 1:
                printf("unchecked");
                break;
            case 4:
                printf("below-low-limit");
                break;
            case 5:
                printf("above-high-limit");
                break;
            default:
                printf("Invalid value in ENUMERATED ((*pData).arr[i11].previous_checking_status)");
            }
            printf(", ");
            printf("current-checking-status ");
            switch((*pData).arr[i11].current_checking_status) {
            case 0:
                printf("witin-limits");
                break;
            case 1:
                printf("unchecked");
                break;
            case 4:
                printf("below-low-limit");
                break;
            case 5:
                printf("above-high-limit");
                break;
            default:
                printf("Invalid value in ENUMERATED ((*pData).arr[i11].current_checking_status)");
            }
            printf(", ");
            printf("transition-time ");
            printf("{");
            printf("p-field ");
            printf("{");
            printf("coarse-time-rep ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).arr[i11].transition_time.p_field.coarse_time_rep);
            #else
            printf("%d", (*pData).arr[i11].transition_time.p_field.coarse_time_rep);
            #endif
            printf(", ");
            printf("fine-time-rep ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).arr[i11].transition_time.p_field.fine_time_rep);
            #else
            printf("%d", (*pData).arr[i11].transition_time.p_field.fine_time_rep);
            #endif
            printf("}");
            printf(", ");
            printf("coarse-time ");
            {
                int i;
                printf("'");
                for(i=0; i<4; i++)
                    printf("%02x", (*pData).arr[i11].transition_time.coarse_time.arr[i]);
                printf("'H");
            }

            printf(", ");
            printf("fine-time ");
            {
                int i;
                printf("'");
                for(i=0; i<3; i++)
                    printf("%02x", (*pData).arr[i11].transition_time.fine_time.arr[i]);
                printf("'H");
            }

            printf("}");
            printf("}");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Var_Character_String(const char *paramName, const asn1SccVar_Character_String *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Var-Character-String ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<(*pData).nCount; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Successful_Telecommand_Verification_Report(const char *paramName, const asn1SccSuccessful_Telecommand_Verification_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Successful-Telecommand-Verification-Report ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("packet-id ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).packet_id);
    #else
    printf("%d", (*pData).packet_id);
    #endif
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_1(const char *paramName, const asn1SccFixed_Bit_String_1 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-1 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i12;
        printf("{");
        for(i12=0; i12<1; i12++) {
            if (i12) 
                printf(",");
            printf("%s", (int)(*pData).arr[i12]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Parameter_ID_Set(const char *paramName, const asn1SccParameter_ID_Set *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Parameter-ID-Set ::= ", paramName);
    printf("%s ", paramName);
    {
        int i13;
        printf("{");
        for(i13=0; i13<(*pData).nCount; i13++) {
            if (i13) 
                printf(",");
            switch((*pData).arr[i13]) {
            case 0:
                printf("voltage-1");
                break;
            case 1:
                printf("voltage-2");
                break;
            default:
                printf("Invalid value in ENUMERATED ((*pData).arr[i13])");
            }
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Verification_Failure_Code(const char *paramName, const asn1SccVerification_Failure_Code *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Verification-Failure-Code ::= ", paramName);
    printf("%s ", paramName);
    #if WORD_SIZE==8
    printf("%lld", (*pData));
    #else
    printf("%d", (*pData));
    #endif
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Time_Rep_Spec(const char *paramName, const asn1SccTime_Rep_Spec *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Time-Rep-Spec ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("coarse-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).coarse_time_rep);
    #else
    printf("%d", (*pData).coarse_time_rep);
    #endif
    printf(", ");
    printf("fine-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).fine_time_rep);
    #else
    printf("%d", (*pData).fine_time_rep);
    #endif
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Var_Octet_String(const char *paramName, const asn1SccVar_Octet_String *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Var-Octet-String ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<(*pData).nCount; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Parameter_Value_Set(const char *paramName, const asn1SccParameter_Value_Set *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Parameter-Value-Set ::= ", paramName);
    printf("%s ", paramName);
    {
        int i14;
        printf("{");
        for(i14=0; i14<(*pData).nCount; i14++) {
            if (i14) 
                printf(",");
            if ((*pData).arr[i14].kind == type_1_0_PRESENT) {
                printf("type-1-0:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i14].u.type_1_0);
                #else
                printf("%d", (*pData).arr[i14].u.type_1_0);
                #endif
            }
            else if ((*pData).arr[i14].kind == type_1_1_PRESENT) {
                printf("type-1-1:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i14].u.type_1_1);
                #else
                printf("%d", (*pData).arr[i14].u.type_1_1);
                #endif
            }
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1On_Board_Fine_Time(const char *paramName, const asn1SccOn_Board_Fine_Time *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s On-Board-Fine-Time ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Checking_Status(const char *paramName, const asn1SccChecking_Status *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Checking-Status ::= ", paramName);
    printf("%s ", paramName);
    switch((*pData)) {
    case 0:
        printf("witin-limits");
        break;
    case 1:
        printf("unchecked");
        break;
    case 4:
        printf("below-low-limit");
        break;
    case 5:
        printf("above-high-limit");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData))");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1On_Board_Coarse_Time(const char *paramName, const asn1SccOn_Board_Coarse_Time *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s On-Board-Coarse-Time ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Real_1(const char *paramName, const asn1SccReal_1 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Real-1 ::= ", paramName);
    printf("%s ", paramName);
    printf("%f", (*pData));
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Parameter_ID(const char *paramName, const asn1SccParameter_ID *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Parameter-ID ::= ", paramName);
    printf("%s ", paramName);
    switch((*pData)) {
    case 0:
        printf("voltage-1");
        break;
    case 1:
        printf("voltage-2");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData))");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Real_2(const char *paramName, const asn1SccReal_2 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Real-2 ::= ", paramName);
    printf("%s ", paramName);
    printf("%f", (*pData));
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Service_Report(const char *paramName, const asn1SccService_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Service-Report ::= ", paramName);
    printf("%s ", paramName);
    if ((*pData).kind == event_reporting_PRESENT) {
        printf("event-reporting:");
        if ((*pData).u.event_reporting.kind == normal_PRESENT) {
            printf("normal:");
            printf("{");
            printf("rid ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.event_reporting.u.normal.rid);
            #else
            printf("%d", (*pData).u.event_reporting.u.normal.rid);
            #endif
            printf("}");
        }
        else if ((*pData).u.event_reporting.kind == anomaly_PRESENT) {
            printf("anomaly:");
            if ((*pData).u.event_reporting.u.anomaly.kind == low_PRESENT) {
                printf("low:");
                printf("{");
                printf("rid ");
                #if WORD_SIZE==8
                printf("%lld", (*pData).u.event_reporting.u.anomaly.u.low.rid);
                #else
                printf("%d", (*pData).u.event_reporting.u.anomaly.u.low.rid);
                #endif
                printf("}");
            }
            else if ((*pData).u.event_reporting.u.anomaly.kind == medium_PRESENT) {
                printf("medium:");
                printf("{");
                printf("rid ");
                #if WORD_SIZE==8
                printf("%lld", (*pData).u.event_reporting.u.anomaly.u.medium.rid);
                #else
                printf("%d", (*pData).u.event_reporting.u.anomaly.u.medium.rid);
                #endif
                printf("}");
            }
            else if ((*pData).u.event_reporting.u.anomaly.kind == high_PRESENT) {
                printf("high:");
                printf("{");
                printf("rid ");
                #if WORD_SIZE==8
                printf("%lld", (*pData).u.event_reporting.u.anomaly.u.high.rid);
                #else
                printf("%d", (*pData).u.event_reporting.u.anomaly.u.high.rid);
                #endif
                printf("}");
            }
        }
    }
    else if ((*pData).kind == telecommand_verification_PRESENT) {
        printf("telecommand-verification:");
        if ((*pData).u.telecommand_verification.kind == success_PRESENT) {
            printf("success:");
            printf("{");
            printf("packet-id ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.telecommand_verification.u.success.packet_id);
            #else
            printf("%d", (*pData).u.telecommand_verification.u.success.packet_id);
            #endif
            printf("}");
        }
        else if ((*pData).u.telecommand_verification.kind == failure_PRESENT) {
            printf("failure:");
            printf("{");
            printf("packet-id ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.telecommand_verification.u.failure.packet_id);
            #else
            printf("%d", (*pData).u.telecommand_verification.u.failure.packet_id);
            #endif
            printf(", ");
            printf("code ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.telecommand_verification.u.failure.code);
            #else
            printf("%d", (*pData).u.telecommand_verification.u.failure.code);
            #endif
            printf("}");
        }
    }
    else if ((*pData).kind == onboard_monitoring_PRESENT) {
        printf("onboard-monitoring:");
        if ((*pData).u.onboard_monitoring.kind == transition_PRESENT) {
            printf("transition:");
            {
                int i15;
                printf("{");
                for(i15=0; i15<(*pData).u.onboard_monitoring.u.transition.nCount; i15++) {
                    if (i15) 
                        printf(",");
                    printf("{");
                    printf("id ");
                    switch((*pData).u.onboard_monitoring.u.transition.arr[i15].id) {
                    case 0:
                        printf("voltage-1");
                        break;
                    case 1:
                        printf("voltage-2");
                        break;
                    default:
                        printf("Invalid value in ENUMERATED ((*pData).u.onboard_monitoring.u.transition.arr[i15].id)");
                    }
                    printf(", ");
                    printf("value ");
                    if ((*pData).u.onboard_monitoring.u.transition.arr[i15].value.kind == type_1_0_PRESENT) {
                        printf("type-1-0:");
                        #if WORD_SIZE==8
                        printf("%lld", (*pData).u.onboard_monitoring.u.transition.arr[i15].value.u.type_1_0);
                        #else
                        printf("%d", (*pData).u.onboard_monitoring.u.transition.arr[i15].value.u.type_1_0);
                        #endif
                    }
                    else if ((*pData).u.onboard_monitoring.u.transition.arr[i15].value.kind == type_1_1_PRESENT) {
                        printf("type-1-1:");
                        #if WORD_SIZE==8
                        printf("%lld", (*pData).u.onboard_monitoring.u.transition.arr[i15].value.u.type_1_1);
                        #else
                        printf("%d", (*pData).u.onboard_monitoring.u.transition.arr[i15].value.u.type_1_1);
                        #endif
                    }
                    printf(", ");
                    printf("limit-crossed ");
                    if ((*pData).u.onboard_monitoring.u.transition.arr[i15].limit_crossed.kind == type_1_0_PRESENT) {
                        printf("type-1-0:");
                        #if WORD_SIZE==8
                        printf("%lld", (*pData).u.onboard_monitoring.u.transition.arr[i15].limit_crossed.u.type_1_0);
                        #else
                        printf("%d", (*pData).u.onboard_monitoring.u.transition.arr[i15].limit_crossed.u.type_1_0);
                        #endif
                    }
                    else if ((*pData).u.onboard_monitoring.u.transition.arr[i15].limit_crossed.kind == type_1_1_PRESENT) {
                        printf("type-1-1:");
                        #if WORD_SIZE==8
                        printf("%lld", (*pData).u.onboard_monitoring.u.transition.arr[i15].limit_crossed.u.type_1_1);
                        #else
                        printf("%d", (*pData).u.onboard_monitoring.u.transition.arr[i15].limit_crossed.u.type_1_1);
                        #endif
                    }
                    printf(", ");
                    printf("previous-checking-status ");
                    switch((*pData).u.onboard_monitoring.u.transition.arr[i15].previous_checking_status) {
                    case 0:
                        printf("witin-limits");
                        break;
                    case 1:
                        printf("unchecked");
                        break;
                    case 4:
                        printf("below-low-limit");
                        break;
                    case 5:
                        printf("above-high-limit");
                        break;
                    default:
                        printf("Invalid value in ENUMERATED ((*pData).u.onboard_monitoring.u.transition.arr[i15].previous_checking_status)");
                    }
                    printf(", ");
                    printf("current-checking-status ");
                    switch((*pData).u.onboard_monitoring.u.transition.arr[i15].current_checking_status) {
                    case 0:
                        printf("witin-limits");
                        break;
                    case 1:
                        printf("unchecked");
                        break;
                    case 4:
                        printf("below-low-limit");
                        break;
                    case 5:
                        printf("above-high-limit");
                        break;
                    default:
                        printf("Invalid value in ENUMERATED ((*pData).u.onboard_monitoring.u.transition.arr[i15].current_checking_status)");
                    }
                    printf(", ");
                    printf("transition-time ");
                    printf("{");
                    printf("p-field ");
                    printf("{");
                    printf("coarse-time-rep ");
                    #if WORD_SIZE==8
                    printf("%lld", (*pData).u.onboard_monitoring.u.transition.arr[i15].transition_time.p_field.coarse_time_rep);
                    #else
                    printf("%d", (*pData).u.onboard_monitoring.u.transition.arr[i15].transition_time.p_field.coarse_time_rep);
                    #endif
                    printf(", ");
                    printf("fine-time-rep ");
                    #if WORD_SIZE==8
                    printf("%lld", (*pData).u.onboard_monitoring.u.transition.arr[i15].transition_time.p_field.fine_time_rep);
                    #else
                    printf("%d", (*pData).u.onboard_monitoring.u.transition.arr[i15].transition_time.p_field.fine_time_rep);
                    #endif
                    printf("}");
                    printf(", ");
                    printf("coarse-time ");
                    {
                        int i;
                        printf("'");
                        for(i=0; i<4; i++)
                            printf("%02x", (*pData).u.onboard_monitoring.u.transition.arr[i15].transition_time.coarse_time.arr[i]);
                        printf("'H");
                    }

                    printf(", ");
                    printf("fine-time ");
                    {
                        int i;
                        printf("'");
                        for(i=0; i<3; i++)
                            printf("%02x", (*pData).u.onboard_monitoring.u.transition.arr[i15].transition_time.fine_time.arr[i]);
                        printf("'H");
                    }

                    printf("}");
                    printf("}");
                }
                printf("}");
            }
        }
    }
    else if ((*pData).kind == event_action_PRESENT) {
        printf("event-action:");
        if ((*pData).u.event_action.kind == foo_PRESENT) {
            printf("foo:");
            #if WORD_SIZE==8
            printf("%lld", (*pData).u.event_action.u.foo);
            #else
            printf("%d", (*pData).u.event_action.u.foo);
            #endif
        }
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_16(const char *paramName, const asn1SccFixed_Bit_String_16 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-16 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i16;
        printf("{");
        for(i16=0; i16<16; i16++) {
            if (i16) 
                printf(",");
            printf("%s", (int)(*pData).arr[i16]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_15(const char *paramName, const asn1SccFixed_Bit_String_15 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-15 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i17;
        printf("{");
        for(i17=0; i17<15; i17++) {
            if (i17) 
                printf(",");
            printf("%s", (int)(*pData).arr[i17]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_14(const char *paramName, const asn1SccFixed_Bit_String_14 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-14 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i18;
        printf("{");
        for(i18=0; i18<14; i18++) {
            if (i18) 
                printf(",");
            printf("%s", (int)(*pData).arr[i18]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_13(const char *paramName, const asn1SccFixed_Bit_String_13 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-13 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i19;
        printf("{");
        for(i19=0; i19<13; i19++) {
            if (i19) 
                printf(",");
            printf("%s", (int)(*pData).arr[i19]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_12(const char *paramName, const asn1SccFixed_Bit_String_12 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-12 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i20;
        printf("{");
        for(i20=0; i20<12; i20++) {
            if (i20) 
                printf(",");
            printf("%s", (int)(*pData).arr[i20]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_11(const char *paramName, const asn1SccFixed_Bit_String_11 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-11 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i21;
        printf("{");
        for(i21=0; i21<11; i21++) {
            if (i21) 
                printf(",");
            printf("%s", (int)(*pData).arr[i21]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Bit_String_10(const char *paramName, const asn1SccFixed_Bit_String_10 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Bit-String-10 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i22;
        printf("{");
        for(i22=0; i22<10; i22++) {
            if (i22) 
                printf(",");
            printf("%s", (int)(*pData).arr[i22]?"TRUE":"FALSE");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Check_Transition(const char *paramName, const asn1SccCheck_Transition *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Check-Transition ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("id ");
    switch((*pData).id) {
    case 0:
        printf("voltage-1");
        break;
    case 1:
        printf("voltage-2");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).id)");
    }
    printf(", ");
    printf("value ");
    if ((*pData).value.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).value.u.type_1_0);
        #else
        printf("%d", (*pData).value.u.type_1_0);
        #endif
    }
    else if ((*pData).value.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).value.u.type_1_1);
        #else
        printf("%d", (*pData).value.u.type_1_1);
        #endif
    }
    printf(", ");
    printf("limit-crossed ");
    if ((*pData).limit_crossed.kind == type_1_0_PRESENT) {
        printf("type-1-0:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).limit_crossed.u.type_1_0);
        #else
        printf("%d", (*pData).limit_crossed.u.type_1_0);
        #endif
    }
    else if ((*pData).limit_crossed.kind == type_1_1_PRESENT) {
        printf("type-1-1:");
        #if WORD_SIZE==8
        printf("%lld", (*pData).limit_crossed.u.type_1_1);
        #else
        printf("%d", (*pData).limit_crossed.u.type_1_1);
        #endif
    }
    printf(", ");
    printf("previous-checking-status ");
    switch((*pData).previous_checking_status) {
    case 0:
        printf("witin-limits");
        break;
    case 1:
        printf("unchecked");
        break;
    case 4:
        printf("below-low-limit");
        break;
    case 5:
        printf("above-high-limit");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).previous_checking_status)");
    }
    printf(", ");
    printf("current-checking-status ");
    switch((*pData).current_checking_status) {
    case 0:
        printf("witin-limits");
        break;
    case 1:
        printf("unchecked");
        break;
    case 4:
        printf("below-low-limit");
        break;
    case 5:
        printf("above-high-limit");
        break;
    default:
        printf("Invalid value in ENUMERATED ((*pData).current_checking_status)");
    }
    printf(", ");
    printf("transition-time ");
    printf("{");
    printf("p-field ");
    printf("{");
    printf("coarse-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).transition_time.p_field.coarse_time_rep);
    #else
    printf("%d", (*pData).transition_time.p_field.coarse_time_rep);
    #endif
    printf(", ");
    printf("fine-time-rep ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).transition_time.p_field.fine_time_rep);
    #else
    printf("%d", (*pData).transition_time.p_field.fine_time_rep);
    #endif
    printf("}");
    printf(", ");
    printf("coarse-time ");
    {
        int i;
        printf("'");
        for(i=0; i<4; i++)
            printf("%02x", (*pData).transition_time.coarse_time.arr[i]);
        printf("'H");
    }

    printf(", ");
    printf("fine-time ");
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).transition_time.fine_time.arr[i]);
        printf("'H");
    }

    printf("}");
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Time_Component_Rep_Spec(const char *paramName, const asn1SccTime_Component_Rep_Spec *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Time-Component-Rep-Spec ::= ", paramName);
    printf("%s ", paramName);
    #if WORD_SIZE==8
    printf("%lld", (*pData));
    #else
    printf("%d", (*pData));
    #endif
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Octet_String_16(const char *paramName, const asn1SccFixed_Octet_String_16 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Octet-String-16 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<16; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Fixed_Character_String_3(const char *paramName, const asn1SccFixed_Character_String_3 *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Fixed-Character-String-3 ::= ", paramName);
    printf("%s ", paramName);
    {
        int i;
        printf("'");
        for(i=0; i<3; i++)
            printf("%02x", (*pData).arr[i]);
        printf("'H");
    }

#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Normal_Progress_Report(const char *paramName, const asn1SccNormal_Progress_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Normal-Progress-Report ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("rid ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).rid);
    #else
    printf("%d", (*pData).rid);
    #endif
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Check_Transition_Set(const char *paramName, const asn1SccCheck_Transition_Set *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Check-Transition-Set ::= ", paramName);
    printf("%s ", paramName);
    {
        int i23;
        printf("{");
        for(i23=0; i23<(*pData).nCount; i23++) {
            if (i23) 
                printf(",");
            printf("{");
            printf("id ");
            switch((*pData).arr[i23].id) {
            case 0:
                printf("voltage-1");
                break;
            case 1:
                printf("voltage-2");
                break;
            default:
                printf("Invalid value in ENUMERATED ((*pData).arr[i23].id)");
            }
            printf(", ");
            printf("value ");
            if ((*pData).arr[i23].value.kind == type_1_0_PRESENT) {
                printf("type-1-0:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i23].value.u.type_1_0);
                #else
                printf("%d", (*pData).arr[i23].value.u.type_1_0);
                #endif
            }
            else if ((*pData).arr[i23].value.kind == type_1_1_PRESENT) {
                printf("type-1-1:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i23].value.u.type_1_1);
                #else
                printf("%d", (*pData).arr[i23].value.u.type_1_1);
                #endif
            }
            printf(", ");
            printf("limit-crossed ");
            if ((*pData).arr[i23].limit_crossed.kind == type_1_0_PRESENT) {
                printf("type-1-0:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i23].limit_crossed.u.type_1_0);
                #else
                printf("%d", (*pData).arr[i23].limit_crossed.u.type_1_0);
                #endif
            }
            else if ((*pData).arr[i23].limit_crossed.kind == type_1_1_PRESENT) {
                printf("type-1-1:");
                #if WORD_SIZE==8
                printf("%lld", (*pData).arr[i23].limit_crossed.u.type_1_1);
                #else
                printf("%d", (*pData).arr[i23].limit_crossed.u.type_1_1);
                #endif
            }
            printf(", ");
            printf("previous-checking-status ");
            switch((*pData).arr[i23].previous_checking_status) {
            case 0:
                printf("witin-limits");
                break;
            case 1:
                printf("unchecked");
                break;
            case 4:
                printf("below-low-limit");
                break;
            case 5:
                printf("above-high-limit");
                break;
            default:
                printf("Invalid value in ENUMERATED ((*pData).arr[i23].previous_checking_status)");
            }
            printf(", ");
            printf("current-checking-status ");
            switch((*pData).arr[i23].current_checking_status) {
            case 0:
                printf("witin-limits");
                break;
            case 1:
                printf("unchecked");
                break;
            case 4:
                printf("below-low-limit");
                break;
            case 5:
                printf("above-high-limit");
                break;
            default:
                printf("Invalid value in ENUMERATED ((*pData).arr[i23].current_checking_status)");
            }
            printf(", ");
            printf("transition-time ");
            printf("{");
            printf("p-field ");
            printf("{");
            printf("coarse-time-rep ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).arr[i23].transition_time.p_field.coarse_time_rep);
            #else
            printf("%d", (*pData).arr[i23].transition_time.p_field.coarse_time_rep);
            #endif
            printf(", ");
            printf("fine-time-rep ");
            #if WORD_SIZE==8
            printf("%lld", (*pData).arr[i23].transition_time.p_field.fine_time_rep);
            #else
            printf("%d", (*pData).arr[i23].transition_time.p_field.fine_time_rep);
            #endif
            printf("}");
            printf(", ");
            printf("coarse-time ");
            {
                int i;
                printf("'");
                for(i=0; i<4; i++)
                    printf("%02x", (*pData).arr[i23].transition_time.coarse_time.arr[i]);
                printf("'H");
            }

            printf(", ");
            printf("fine-time ");
            {
                int i;
                printf("'");
                for(i=0; i<3; i++)
                    printf("%02x", (*pData).arr[i23].transition_time.fine_time.arr[i]);
                printf("'H");
            }

            printf("}");
            printf("}");
        }
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Failing_Telecommand_Verification_Report(const char *paramName, const asn1SccFailing_Telecommand_Verification_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Failing-Telecommand-Verification-Report ::= ", paramName);
    printf("%s ", paramName);
    printf("{");
    printf("packet-id ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).packet_id);
    #else
    printf("%d", (*pData).packet_id);
    #endif
    printf(", ");
    printf("code ");
    #if WORD_SIZE==8
    printf("%lld", (*pData).code);
    #else
    printf("%d", (*pData).code);
    #endif
    printf("}");
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

void PrintASN1Error_Anomaly_Report(const char *paramName, const asn1SccError_Anomaly_Report *pData)
{
#ifdef __linux__
    pthread_mutex_lock(&g_printing_mutex);
#endif
    //printf("%s Error-Anomaly-Report ::= ", paramName);
    printf("%s ", paramName);
    if ((*pData).kind == low_PRESENT) {
        printf("low:");
        printf("{");
        printf("rid ");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.low.rid);
        #else
        printf("%d", (*pData).u.low.rid);
        #endif
        printf("}");
    }
    else if ((*pData).kind == medium_PRESENT) {
        printf("medium:");
        printf("{");
        printf("rid ");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.medium.rid);
        #else
        printf("%d", (*pData).u.medium.rid);
        #endif
        printf("}");
    }
    else if ((*pData).kind == high_PRESENT) {
        printf("high:");
        printf("{");
        printf("rid ");
        #if WORD_SIZE==8
        printf("%lld", (*pData).u.high.rid);
        #else
        printf("%d", (*pData).u.high.rid);
        #endif
        printf("}");
    }
#ifdef __linux__
    pthread_mutex_unlock(&g_printing_mutex);
#endif
}

