#!/bin/bash

# This script will build your TASTE system.

# You must check it before running it: it may need to be adapted to your context:
# 1) You may need to fix some paths and filenames (path to interface/deployment views)
# 2) You may need to specify additional paths for the compiler to find .h file
#    (e.g. "export C_INCLUDE_PATH=/usr/include/xenomai/analogy/:$C_INCLUDE_PATH")
# 3) You may need to link with pre-built libraries, using the -l option
#    (e.g. -l /usr/lib/libanalogy.a,/usr/lib/librtdm.a)
# 4) You may need to change the runtime (add -p flag to select PolyORB-HI-C)
# etc.

# Note: TASTE will not overwrite your changes - if you need to update some  parts
#       you will have to merge the changes with the newly-created "build-script.new.sh" file.

if [ -z "$DEPLOYMENTVIEW" ]
then
    DEPLOYMENTVIEW=DeploymentView.aadl
fi

SKELS="./"

cd "$SKELS"
rm -f thechecker.zip
zip thechecker thechecker/*
rm -f associatedcheckstates.zip
zip associatedcheckstates associatedcheckstates/*
rm -f associatedcheckdefinitions.zip
zip associatedcheckdefinitions associatedcheckdefinitions/*
rm -f thegenerator.zip
zip thegenerator thegenerator/*
rm -f thechecktransitionreport.zip
zip thechecktransitionreport thechecktransitionreport/*
rm -f reportedcheckstatetransitions.zip
zip reportedcheckstatetransitions reportedcheckstatetransitions/*
rm -f checkedparameters.zip
zip checkedparameters checkedparameters/*
rm -f applicationprocess.zip
zip applicationprocess applicationprocess/*
rm -f parameterpool.zip
zip parameterpool parameterpool/*
rm -f missiondatabase.zip
zip missiondatabase missiondatabase/*
rm -f ground.zip
zip ground ground/*
cd "$OLDPWD"

[ ! -z "$CLEANUP" ] && rm -rf binary

echo 'Building the system with the Ada runtime (add -p in the build script to replace with C)'
assert-builder-ocarina.py \
	-f \
	-g \
	--aadlv2 \
	--keep-case \
	--interfaceView InterfaceView.aadl \
	--deploymentView "$DEPLOYMENTVIEW" \
	-o binary \
	--subAda thechecker:"$SKELS"/thechecker.zip \
	--subAda associatedcheckstates:"$SKELS"/associatedcheckstates.zip \
	--subAda associatedcheckdefinitions:"$SKELS"/associatedcheckdefinitions.zip \
	--subAda thegenerator:"$SKELS"/thegenerator.zip \
	--subAda thechecktransitionreport:"$SKELS"/thechecktransitionreport.zip \
	--subAda reportedcheckstatetransitions:"$SKELS"/reportedcheckstatetransitions.zip \
	--subAda checkedparameters:"$SKELS"/checkedparameters.zip \
	--subAda applicationprocess:"$SKELS"/applicationprocess.zip \
	--subAda parameterpool:"$SKELS"/parameterpool.zip \
	--subAda missiondatabase:"$SKELS"/missiondatabase.zip \
	--subAda ground:"$SKELS"/ground.zip