pragma Style_Checks (Off); -- turn off style checks
WITH Interfaces;
USE Interfaces;



WITH Unchecked_Conversion;

PACKAGE AdaAsn1RTL IS



   TYPE BIT IS mod 2**1;
   TYPE BitArray IS ARRAY (Natural  RANGE <>) OF BIT;
   for BitArray'Component_Size use 1;
   pragma Pack(BitArray);

   TYPE OctetBuffer IS ARRAY (Natural  RANGE <>) OF Unsigned_8;
   TYPE OctetBuffer_Ptr IS ACCESS ALL OctetBuffer;

   SUBTYPE Asn1Int IS Long_Long_Integer;
   SUBTYPE Asn1UInt IS Unsigned_64;
   MSBIT_ONE  : CONSTANT Asn1UInt := 2**(Asn1UInt'SIZE - 1);
   MSBYTE_FF  : CONSTANT Asn1UInt := Shift_Left(Asn1UInt(16#FF#), Asn1UInt'Size-8);


   TYPE Asn1SccSint IS NEW Asn1Int;

   SUBTYPE Octet IS BitArray(1..8);

   FUNCTION Octet_to_byte(c:BitArray) RETURN Unsigned_8;
   FUNCTION Byte_to_octet(b:Unsigned_8) RETURN Octet;





   SUBTYPE Asn1Real IS LONG_Float;
   MantissaFactor : Asn1Real:=Asn1Real(Unsigned_64(2)**Asn1Real'Machine_Mantissa);


   TYPE OctetArray4 IS NEW OctetBuffer(1..4);
   TYPE OctetArray8 IS NEW OctetBuffer(1..8);

   FUNCTION Float_to_OctetArray4 IS NEW Unchecked_Conversion(Float, OctetArray4);
   FUNCTION Long_Float_to_OctetArray8 IS NEW Unchecked_Conversion(Asn1Real, OctetArray8);
   FUNCTION OctetArray4_to_Float IS NEW Unchecked_Conversion(OctetArray4, Float);
   FUNCTION OctetArray8_to_Long_Float IS NEW Unchecked_Conversion(OctetArray8, Asn1Real);


   TYPE Asn1Boolean IS NEW Boolean;
   FOR Asn1Boolean'SIZE USE 32;


   FUNCTION Ternary_Booleam(a,b,c: Boolean) return boolean;


   SUBTYPE Flag IS Asn1Boolean;

   TYPE Asn1NullType IS NEW Character range Character'Val(0)..Character'Val(0);

   FUNCTION Strlen(Str:IN STRING) RETURN INTEGER;

   FUNCTION Strchr(Str:IN STRING; Ch:IN Character) RETURN BOOLEAN;

   FUNCTION GetZeroBasedCharIndex(CharToSearch:IN Character; AllowedCharSet:IN String) RETURN Asn1Int;

   ERR_INSUFFICIENT_DATA          : CONSTANT INTEGER := 101;
   ERR_INCORRECT_PER_STREAM       : CONSTANT INTEGER := 102;
   ERR_INVALID_CHOICE_ALTERNATIVE : CONSTANT INTEGER := 103;
   ERR_INCORRECT_STREAM		  : CONSTANT INTEGER := 104;
   ERR_INVALID_BER_FILE		  : CONSTANT INTEGER := 201;
   ERR_BER_LENGTH_MISMATCH	  : CONSTANT INTEGER := 202;
   ERR_BER_TAG_MISMATCH	  	  : CONSTANT INTEGER := 203;


   FUNCTION GetLengthInBytesOfUInt (V:Asn1Uint) RETURN Integer;
   FUNCTION GetLengthInBytesOfSInt (V:Asn1Int)  RETURN Integer;
   FUNCTION To_UInt(IntVal:Asn1Int) return Asn1Uint;
   FUNCTION UInt_To_Int(uVal:Asn1UInt; uintSize:INTEGER) return Asn1Int;

   GENERIC
      MAX : POSITIVE;
   PACKAGE BIT_STRING_PACKAGE IS
      TYPE BIT_STRING IS
         RECORD
            Length : Integer;
            Data  : BitArray (1 .. MAX);
         END RECORD;

      TYPE FIXED_BIT_STRING IS
         RECORD
            Data  : BitArray (1 .. MAX);
         END RECORD;

   END BIT_STRING_PACKAGE;



   TYPE ASN1_RESULT IS
      RECORD
         Success   : Boolean;
         ErrorCode : INTEGER := 0;
      END RECORD;



   TYPE BitStream
         (N : Integer) IS
      RECORD
         CurrentBit : Natural :=1;
         Data       : BitArray (1 .. N) := (1..N=>0);
      END RECORD;



   PROCEDURE BitStream_Init(Stream:OUT BitStream);
   PROCEDURE BitStream_AttachBuffer(Stream:OUT BitStream; Buf:IN OctetBuffer);
   PROCEDURE BitStream_AppendBit(Strm:IN OUT BitStream; BitVal:IN BIT);
   PROCEDURE BitStream_ReadBit(Strm:IN OUT BitStream; BitVal:OUT BIT; Result:OUT ASN1_RESULT);
   PROCEDURE BitStream_ReadBit(Strm:IN OUT BitStream; BitVal:OUT Asn1Boolean; Result:OUT ASN1_RESULT);

   PROCEDURE Acn_AlignToNextByte(Stream:IN OUT BitStream);
   PROCEDURE Acn_AlignToNextWord(Stream:IN OUT BitStream);
   PROCEDURE Acn_AlignToNextDWord(Stream:IN OUT BitStream);

   PROCEDURE BitStream_AppendBit(Strm:IN OUT BitStream; BitVal:IN Asn1Boolean);
   PROCEDURE BitStream_AppendBitN(Strm:IN OUT BitStream; BitVal:IN BIT; NBits:IN INTEGER);
   PROCEDURE BitStream_AppendBits(Strm:IN OUT BitStream; Buf:IN OctetBuffer; NBits:IN INTEGER);
   PROCEDURE BitStream_AppendBitString(Strm:IN OUT BitStream; Data:IN BitArray);
   PROCEDURE BitStream_ReadBitPattern(Strm:IN OUT BitStream; patternToRead:IN BitArray; patternMatches:OUT Asn1Boolean; Result:OUT ASN1_RESULT);
   PROCEDURE BitStream_ReadBits (Strm:IN OUT BitStream; Data:IN OUT BitArray; Count:IN INTEGER; Result:OUT ASN1_RESULT);

   PROCEDURE BitStream_AppendByte(Strm:IN OUT BitStream; ByteValue:IN Unsigned_8; Negate:IN Boolean);
   PROCEDURE BitStream_ReadByte (Strm:IN OUT BitStream; ByteValue:OUT Unsigned_8; Result:OUT ASN1_RESULT);

   -- nBits 1 .. 7
   PROCEDURE BitStream_AppendPartialByte (Strm:IN OUT BitStream; ByteValue:IN Unsigned_8; NBits:IN INTEGER; Negate:IN Boolean);

   PROCEDURE UPER_Enc_UnConstraintWholeNumber (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE UPER_Dec_UnConstraintWholeNumber (Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE UPER_Enc_SemiConstraintWholeNumber(Strm:IN OUT BitStream; IntVal:IN Asn1Int; MinVal:IN Asn1Int);
   PROCEDURE UPER_Dec_SemiConstraintWholeNumber(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; MinVal:IN Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE UPER_Enc_ConstraintWholeNumber(Strm:IN OUT BitStream; IntVal:IN Asn1Int; MinVal:IN Asn1Int; MaxVal:IN Asn1Int);
   PROCEDURE UPER_Dec_ConstraintWholeNumber (Strm:IN OUT BitStream; IntVal:OUT Asn1Int; MinVal:IN Asn1Int; MaxVal:IN Asn1Int; Result:OUT ASN1_RESULT);

   PROCEDURE UPER_Enc_Real(Strm:IN OUT BitStream; RealVal:IN Asn1Real);
   PROCEDURE UPER_Dec_Real(Strm:IN OUT BitStream; RealVal:OUT Asn1Real; Result:OUT ASN1_RESULT);


   GENERIC
      MAX : POSITIVE;
      TYPE ELEMENT IS PRIVATE;
   PACKAGE SEQUENCE_OF_PACKAGE IS

      TYPE LIST IS ARRAY (Positive RANGE 1 .. MAX) OF ELEMENT;

      TYPE SEQUENCE_OF IS
         RECORD
            Length : Integer;
            Data  : LIST;
         END RECORD;

      TYPE FIXED_SEQUENCE_OF IS
         RECORD
            Data  : LIST;
         END RECORD;

   END SEQUENCE_OF_PACKAGE;



   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize (Strm:IN OUT BitStream; IntVal:IN Asn1Int; Size:IN Integer);
   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize_8 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize_big_endian_16 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize_big_endian_32 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize_big_endian_64 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize_little_endian_16 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize_little_endian_32 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_PositiveInteger_ConstSize_little_endian_64 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);



   PROCEDURE Acn_Enc_Int_PositiveInteger_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:IN Asn1Int);

   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize (Strm:IN OUT BitStream; IntVal:IN Asn1Int; Size:IN Integer);
   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize_8 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize_big_endian_16 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize_big_endian_32 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize_big_endian_64 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize_little_endian_16 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize_little_endian_32 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_TwosComplement_ConstSize_little_endian_64 (Strm:IN OUT BitStream; IntVal:IN Asn1Int);

   PROCEDURE Acn_Enc_Int_TwosComplement_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:IN Asn1Int);

   PROCEDURE Acn_Enc_Int_BCD_ConstSize (Strm:IN OUT BitStream; IntVal:IN Asn1Int; Size:IN Integer);
   PROCEDURE Acn_Enc_Int_BCD_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_BCD_VarSize_NullTerminated(Strm:IN OUT BitStream; IntVal:IN Asn1Int);

   PROCEDURE Acn_Enc_Int_ASCII_ConstSize (Strm:IN OUT BitStream; IntVal:IN Asn1Int; Size:IN Integer);
   PROCEDURE Acn_Enc_Int_ASCII_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:IN Asn1Int);
   PROCEDURE Acn_Enc_Int_ASCII_VarSize_NullTerminated(Strm:IN OUT BitStream; IntVal:IN Asn1Int);

   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize (Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Size:IN Integer; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize_8(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize_big_endian_16(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize_big_endian_32(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize_big_endian_64(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize_little_endian_16(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize_little_endian_32(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_PositiveInteger_ConstSize_little_endian_64(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);

   PROCEDURE Acn_Dec_Int_PositiveInteger_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);

   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize (Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Size:IN Integer; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize_8(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize_big_endian_16(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize_big_endian_32(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize_big_endian_64(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize_little_endian_16(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize_little_endian_32(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_TwosComplement_ConstSize_little_endian_64(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);



   PROCEDURE Acn_Dec_Int_TwosComplement_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);

   PROCEDURE Acn_Dec_Int_BCD_ConstSize (Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Size:IN Integer; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_BCD_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_BCD_VarSize_NullTerminated(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);

   PROCEDURE Acn_Dec_Int_ASCII_ConstSize (Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Size:IN Integer; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_ASCII_VarSize_LengthEmbedded(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Dec_Int_ASCII_VarSize_NullTerminated(Strm:IN OUT BitStream; IntVal:OUT Asn1Int; Result:OUT ASN1_RESULT);

   PROCEDURE Acn_Enc_Real_IEEE754_32_big_endian(Strm:IN OUT BitStream; RealVal:IN Asn1Real);
   PROCEDURE Acn_Dec_Real_IEEE754_32_big_endian(Strm:IN OUT BitStream; RealVal:OUT Asn1Real; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Enc_Real_IEEE754_64_big_endian(Strm:IN OUT BitStream; RealVal:IN Asn1Real);
   PROCEDURE Acn_Dec_Real_IEEE754_64_big_endian(Strm:IN OUT BitStream; RealVal:OUT Asn1Real; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Enc_Real_IEEE754_32_little_endian(Strm:IN OUT BitStream; RealVal:IN Asn1Real);
   PROCEDURE Acn_Dec_Real_IEEE754_32_little_endian(Strm:IN OUT BitStream; RealVal:OUT Asn1Real; Result:OUT ASN1_RESULT);
   PROCEDURE Acn_Enc_Real_IEEE754_64_little_endian(Strm:IN OUT BitStream; RealVal:IN Asn1Real);
   PROCEDURE Acn_Dec_Real_IEEE754_64_little_endian(Strm:IN OUT BitStream; RealVal:OUT Asn1Real; Result:OUT ASN1_RESULT);



END AdaAsn1RTL;
